from rest_framework import exceptions
from rest_framework.permissions import AllowAny
from rest_framework.renderers import CoreJSONRenderer
from rest_framework.response import Response
from rest_framework.schemas import SchemaGenerator
from rest_framework.views import APIView

from . import renderers


def get_swagger_view(title=None, url=None, patterns=None, urlconf=None):
    """
    Returns schema view which renders Swagger/OpenAPI.
    """
    class SwaggerSchemaView(APIView):
        _ignore_model_permissions = True
        exclude_from_schema = True
        permission_classes = [AllowAny]
        renderer_classes = [
            CoreJSONRenderer,
            renderers.OpenAPIRenderer,
            renderers.SwaggerUIRenderer
        ]

        def get(self, request):
            generator = SchemaGenerator(
                title=title,
                url=url,
                patterns=patterns,
                urlconf=urlconf
            )
            schema = generator.get_schema(request=request)

            if not schema:
                raise exceptions.ValidationError(
                    'The schema generator did not return a schema Document'
                )

            return Response(schema)

    return SwaggerSchemaView.as_view()



from django.conf import settings
from django.test.signals import setting_changed
from rest_framework.settings import APISettings


DEFAULTS = {
    'USE_SESSION_AUTH': True,
    'SECURITY_DEFINITIONS': {
        'basic': {
            'type': 'basic'
        }
    },
    'LOGIN_URL': getattr(settings, 'LOGIN_URL', None),
    'LOGOUT_URL': getattr(settings, 'LOGOUT_URL', None),
    'DOC_EXPANSION': None,
    'APIS_SORTER': None,
    'OPERATIONS_SORTER': None,
    'JSON_EDITOR': False,
    'SHOW_REQUEST_HEADERS': False,
    'SUPPORTED_SUBMIT_METHODS': [
        'get',
        'post',
        'put',
        'delete',
        'patch'
    ],
    'VALIDATOR_URL': '',
    'ACCEPT_HEADER_VERSION': None,  # e.g. '1.0'
    'CUSTOM_HEADERS': {}  # A dictionary of key/vals to override headers
}

IMPORT_STRINGS = []

swagger_settings = APISettings(
    user_settings=getattr(settings, 'SWAGGER_SETTINGS', {}),
    defaults=DEFAULTS,
    import_strings=IMPORT_STRINGS
)


def reload_settings(*args, **kwargs):  # pragma: no cover
    """
    Reloads settings during unit tests if override_settings decorator
    is used. (Taken from DRF)
    """
    # pylint: disable=W0603
    global swagger_settings

    if kwargs['setting'] == 'LOGIN_URL':
        swagger_settings.LOGIN_URL = kwargs['value']
    if kwargs['setting'] == 'LOGOUT_URL':
        swagger_settings.LOGOUT_URL = kwargs['value']
    if kwargs['setting'] != 'SWAGGER_SETTINGS':
        return

    swagger_settings = APISettings(
        kwargs['value'],
        DEFAULTS,
        IMPORT_STRINGS
    )


setting_changed.connect(reload_settings)



import coreapi
from coreapi.compat import force_bytes
from django.shortcuts import render, resolve_url
from openapi_codec import OpenAPICodec as _OpenAPICodec
from openapi_codec.encode import generate_swagger_object
from rest_framework.renderers import BaseRenderer, JSONRenderer
from rest_framework import status
import simplejson as json

from .settings import swagger_settings as settings


class OpenAPICodec(_OpenAPICodec):
    def encode(self, document, **options):
        if not isinstance(document, coreapi.Document):
            raise TypeError('Expected a `coreapi.Document` instance')

        data = generate_swagger_object(document)
        data.update(**options)

        return force_bytes(json.dumps(data))


class OpenAPIRenderer(BaseRenderer):
    media_type = 'application/openapi+json'
    charset = None
    format = 'openapi'

    def render(self, data, accepted_media_type=None, renderer_context=None):
        if renderer_context['response'].status_code != status.HTTP_200_OK:
            return JSONRenderer().render(data)
        options = self.get_customizations()

        return OpenAPICodec().encode(data, **options)

    def get_customizations(self):
        """
        Adds settings, overrides, etc. to the specification.
        """
        data = {}
        if settings.SECURITY_DEFINITIONS:
            data['securityDefinitions'] = settings.SECURITY_DEFINITIONS

        return data


class SwaggerUIRenderer(BaseRenderer):
    media_type = 'text/html'
    format = 'swagger'
    template = 'rest_framework_swagger/index.html'
    charset = 'utf-8'

    def render(self, data, accepted_media_type=None, renderer_context=None):
        self.set_context(data, renderer_context)
        return render(
            renderer_context['request'],
            self.template,
            renderer_context
        )

    def set_context(self, data, renderer_context):
        renderer_context['USE_SESSION_AUTH'] = \
            settings.USE_SESSION_AUTH
        renderer_context.update(self.get_auth_urls())

        drs_settings = self.get_ui_settings()
        renderer_context['drs_settings'] = json.dumps(drs_settings)
        renderer_context['spec'] = OpenAPIRenderer().render(
            data=data,
            renderer_context=renderer_context
        ).decode()

    def get_auth_urls(self):
        urls = {}
        if settings.LOGIN_URL is not None:
            urls['LOGIN_URL'] = resolve_url(settings.LOGIN_URL)
        if settings.LOGOUT_URL is not None:
            urls['LOGOUT_URL'] = resolve_url(settings.LOGOUT_URL)

        return urls

    def get_ui_settings(self):
        data = {
            'apisSorter': settings.APIS_SORTER,
            'docExpansion': settings.DOC_EXPANSION,
            'jsonEditor': settings.JSON_EDITOR,
            'operationsSorter': settings.OPERATIONS_SORTER,
            'showRequestHeaders': settings.SHOW_REQUEST_HEADERS,
            'supportedSubmitMethods': settings.SUPPORTED_SUBMIT_METHODS,
            'acceptHeaderVersion': settings.ACCEPT_HEADER_VERSION,
            'customHeaders': settings.CUSTOM_HEADERS,
        }
        if settings.VALIDATOR_URL != '':
            data['validatorUrl'] = settings.VALIDATOR_URL

        return data



#!/usr/bin/python
import os
import sys
import shutil
from setuptools import setup
from rest_framework_swagger import __version__ as VERSION

if sys.argv[-1] == 'publish':
    if os.system("wheel version"):
        print("wheel not installed.\nUse `pip install wheel`.\nExiting.")
        sys.exit()
    if os.system("pip freeze | grep twine"):
        print("twine not installed.\nUse `pip install twine`.\nExiting.")
        sys.exit()
    os.system("python setup.py sdist bdist_wheel")
    os.system("twine upload -r pypi dist/*")
    print("You probably want to also tag the version now:")
    print("  git tag -a %s -m 'version %s'" % (VERSION, VERSION))
    print("  git push --tags")
    shutil.rmtree('dist')
    shutil.rmtree('build')
    shutil.rmtree('django_rest_swagger.egg-info')
    sys.exit()

README = """
Django REST Swagger

An API documentation generator for Swagger UI and Django REST Framework.

Installation
From pip:

pip install django-rest-swagger-django3fix

Project @ https://github.com/marcgibbons/django-rest-swagger
Docs @ https://django-rest-swagger.readthedocs.io/
"""

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-rest-swagger-django3fix',
    version=VERSION,
    install_requires=[
        'coreapi>=2.3.0',
        'openapi-codec>=1.3.1',
        'djangorestframework>=3.5.4',
        'Django>=1.8',
        'simplejson'
    ],
    packages=['rest_framework_swagger'],
    include_package_data=True,
    license='FreeBSD License',
    description='Swagger UI for Django REST Framework 3.5+',
    long_description=README,
    test_suite='tests',
    author='Marc Gibbons',
    author_email='marc_gibbons@rogers.com',
    url='https://github.com/marcgibbons/django-rest-swagger',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 2.1',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    zip_safe=False
)
